local f = CreateFrame("Frame")
local now = "PP: GET IT NOW!"
local ready = "PP: Get to %s: %d sec!"
local target = "PP: %s is taking the plague!"
local raidwarning = "## %s is next! ##"

local tenManDuration = 15
local twentyFiveManDuration = 10
local currentDuration = nil

local possibleTargets = {
	--[[
	"Foo",
	"Bar",
	"Etc",
	]]
	-- Add in all the ranged and healers here,
	-- or whoever you want to use for the plague
	-- handling.
}
if #possibleTargets == 0 then
	print("WARNING: You have not set up a list of possible targets for the Unbound Plague.")
	print("Please edit PlaguePuppets/plague.lua and fill in the possibleTargets list before reloading your interface.")
end

local ids = {
	[72855] = true,
	[72856] = true,
	[70911] = true,
	[72854] = true,
}
local total = 0

local plague = GetSpellInfo(72855)
local sickness = GetSpellInfo(73117)

local nextTarget = nil
local current = nil
local applied = nil

local tmpTimes = {}
local tmpNames = {}
local function sortFunc(a, b) return (tmpTimes[a] or 0) < (tmpTimes[b] or 0) end
local function getNextTarget()
	wipe(tmpTimes)
	wipe(tmpNames)
	local nt = nil
	for i, player in next, possibleTargets do
		if UnitInRaid(player) and current ~= player and not UnitIsDeadOrGhost(player) then
			local name, _, _, _, _, _, expirationTime = UnitDebuff(player, sickness)
			if not name then
				print(player .. " is not debuffed.")
				nt = player
				break
			end
			tmpTimes[player] = expirationTime
			tmpNames[#tmpNames + 1] = player
		end
	end
	if not nt then
		table.sort(tmpNames, sortFunc)
		nt = tmpNames[1]
		print("Found no clean targets, returning " .. nt)
	end
	return nt
end

f:SetScript("OnEvent", function(self, _, _, event, _, _, _, _, player, _, spellId)
	if not event or not player or not spellId or not ids[spellId] then return end
	if event == "SPELL_AURA_APPLIED" then
		if not currentDuration then
			local d = GetInstanceDifficulty()
			currentDuration = d == 3 and tenManDuration or twentyFiveManDuration
			print("Plague duration set to " .. currentDuration)
		end
		nextTarget = nil
		current = player
		applied = GetTime()
		SetRaidTarget(current, 7)
		local name, _, _, _, _, _, expirationTime = UnitDebuff(current, plague)
		if name and expirationTime < (applied + currentDuration) then
			self:Hide()
		else
			print(current .. " got the plague, starting timers")
			self:Show()
		end
	end
end)

f:SetScript("OnUpdate", function(self, elapsed)
	if not current then
		self:Hide()
		return
	end

	total = total + elapsed
	if total >= 1 then
		local name = UnitDebuff(current, plague)
		if not name then
			print(("%s doesn't have the plague?! It just vanished!"):format(current))
			self:Hide()
			return
		end
		local t = GetTime()
		if t > (applied + (currentDuration - 1)) then
			SendChatMessage(now, "WHISPER", nil, nextTarget)
		elseif t > (applied + 4) then
			if not nextTarget then
				nextTarget = getNextTarget()
				SetRaidTarget(nextTarget, 1)
				SendChatMessage(target:format(nextTarget), "WHISPER", nil, current)
				SendChatMessage(raidwarning:format(nextTarget), "RAID_WARNING")
			end
			local tick = currentDuration - (t - applied)
			SendChatMessage(ready:format(current, tick), "WHISPER", nil, nextTarget)
		end
		total = 0
	end
end)
f:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
f:Hide()

local function filter(self, event, msg)
	if msg:find("^PP: ") then return true end
end
ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", filter)

-- Debug
_G.PlagueHelper = f


